package de.uni.karlsruhe.aifb.prog2.chess;

/** Die Springer-Figur des Schachspiels. */
public class Knight extends AbstractChessman {

  /** Konstruktor.
    * @param isBlack ist true genau dann, wenn die Figur schwarz ist.
    **/
  public Knight(boolean isBlack) {
    super(isBlack,'N');
  }

  /** Richtungen, in die sich das Pferd bewegen kann. Der erste
    * Eintrag ist stets die Richtung in der Zeile, die zweite die
    * Richtung in der Spalte.
    **/
  private final static int[][] DIRECTIONS =
    {{-2,-1},{-2,1},{-1,-2},{-1,2},{1,-2},{1,2},{2,-1},{2,1}};

  /** Ueberprueft, auf welche Felder sich die Figur im naechsten Zug
    * theoretisch bewegen kann.
    **/
  public boolean[][] getDestinations
    (Chessboard board,int row,int col) {
    // Bis das Gegenteil bewiesen ist, darf unsere Figur nirgends hin
    boolean[][] res =
      new boolean[board.getNumOfRows()]
                 [board.getNumOfColumns()];
    // In einer Schleife berechnen wir alle Positionen,
    //  die theoretisch moeglich sind.
    for (int i = 0; i < DIRECTIONS.length; i++) {
      int x = row + DIRECTIONS[i][0];
      int y = col + DIRECTIONS[i][1];
      // Sind die Koordinaten im gueltigen Bereich?
      if (board.isValid(x,y))
        res[x][y] = true;
    }
    // Eliminiere nun die Felder, die nicht betreten werden koennen
    cleanBlocked(res,board,this);
    // Gib das Ergebnis zurueck
    return res;
  }
}